//	Visual Clipper Ver 2.0pʃt@C֐
#pragma once

//	ϊs\܂B
//	W(x, y)̈ړ(x', y')ƂƂȉ̎܂B
//	x' = (x * _11 + y * _12 + _13) / (x * _31 + y * _32 + _33)
//	y' = (x * _21 + y * _22 + _23) / (x * _31 + y * _32 + _33)
struct TRANSFORMMATRIX
{
	union
	{
		FLOAT m[3][3];
		struct
		{
			FLOAT _11;
			FLOAT _12;
			FLOAT _13;
			FLOAT _21;
			FLOAT _22;
			FLOAT _23;
			FLOAT _31;
			FLOAT _32;
			FLOAT _33;
		};
	};
	inline FLOAT TransformX(FLOAT x, FLOAT y);
	inline FLOAT TransformY(FLOAT x, FLOAT y);
	inline FLOAT TransformZ(FLOAT x, FLOAT y);
	inline void Transform(FLOAT& px, FLOAT& py);
	inline void Transform(FLOAT& px, FLOAT& py, FLOAT x, FLOAT y);
	inline void TransformInteger(LONG& px, LONG& py);
	inline void TransformInteger(LONG& px, LONG& py, LONG lx, LONG ly);
	inline void TransformIntegerB(LONG& px, LONG& py);
	inline void TransformIntegerB(LONG& px, LONG& py, LONG lx, LONG ly);
	inline void SetInverse();
	inline void SetIdentity();
	inline void MultiplyLeft(TRANSFORMMATRIX* pMatrix);
	inline void MultiplyRight(TRANSFORMMATRIX* pMatrix);
};

#define TM_DOUBLE_EPSILON					(1.0 / 16777216)

inline FLOAT TRANSFORMMATRIX::TransformX(FLOAT x, FLOAT y)
{
	return x * _11 + y * _12 + _13;
}

inline FLOAT TRANSFORMMATRIX::TransformY(FLOAT x, FLOAT y)
{
	return x * _21 + y * _22 + _23;
}

inline FLOAT TRANSFORMMATRIX::TransformZ(FLOAT x, FLOAT y)
{
	return x * _31 + y * _32 + _33;
}

inline void TRANSFORMMATRIX::Transform(FLOAT& px, FLOAT& py)
{
	FLOAT x;
	FLOAT y;
	FLOAT z;
	x = px;
	y = py;
	z = TransformZ(x, y);
	px = TransformX(x, y) / z;
	py = TransformY(x, y) / z;
}

inline void TRANSFORMMATRIX::Transform(FLOAT& px, FLOAT& py, FLOAT x, FLOAT y)
{
	FLOAT z;
	z = TransformZ(x, y);
	px = TransformX(x, y) / z;
	py = TransformY(x, y) / z;
}

inline void TRANSFORMMATRIX::TransformInteger(LONG& px, LONG& py)
{
	FLOAT x;
	FLOAT y;
	FLOAT z;
	x = (FLOAT)px;
	y = (FLOAT)py;
	z = TransformZ(x, y);
	px = (LONG)(TransformX(x, y) / z);
	py = (LONG)(TransformY(x, y) / z);
}

inline void TRANSFORMMATRIX::TransformInteger(LONG& px, LONG& py, LONG lx, LONG ly)
{
	FLOAT x;
	FLOAT y;
	FLOAT z;
	x = (FLOAT)lx;
	y = (FLOAT)ly;
	z = TransformZ(x, y);
	px = (LONG)(TransformX(x, y) / z);
	py = (LONG)(TransformY(x, y) / z);
}

inline void TRANSFORMMATRIX::TransformIntegerB(LONG& px, LONG& py)
{
	FLOAT x;
	FLOAT y;
	FLOAT z;
	x = (FLOAT)px;
	y = (FLOAT)py;
	z = TransformZ(x, y);
	px = (LONG)(TransformX(x, y) / z + 1.0f) - 1;
	py = (LONG)(TransformY(x, y) / z + 1.0f) - 1;
}

inline void TRANSFORMMATRIX::TransformIntegerB(LONG& px, LONG& py, LONG lx, LONG ly)
{
	FLOAT x;
	FLOAT y;
	FLOAT z;
	x = (FLOAT)lx;
	y = (FLOAT)ly;
	z = TransformZ(x, y);
	px = (LONG)(TransformX(x, y) / z + 1.0f) - 1;
	py = (LONG)(TransformY(x, y) / z + 1.0f) - 1;
}

inline void TRANSFORMMATRIX::SetInverse()
{
	DWORD i;
	DWORD j;
	DWORD k;
	DOUBLE f;
	DOUBLE g;
	DOUBLE d[3][3];
	for(i = 0; i < 3; i++)
	{
		for(j = 0; j < 3; j++)
			d[i][j] = m[i][j];
	}
	for(i = 0; i < 3; i++)
	{
		f = d[i][i];
		if(f < TM_DOUBLE_EPSILON && f >= 0.0)
			f = TM_DOUBLE_EPSILON;
		else if(f > -TM_DOUBLE_EPSILON && f < 0.0)
			f = -TM_DOUBLE_EPSILON;
		f = 1.0 / f;
		for(j = 0; j < 3; j++)
			d[i][j] *= f;
		d[i][i] = f;
		for(j = 0; j < 3; j++)
		{
			if(j != i)
			{
				g = d[j][i];
				for(k = 0; k < 3; k++)
				{
					if(k != i)
						d[j][k] -= d[i][k] * g;
					else
						d[j][k] = -f * g;
				}
			}
		}
	}
	for(i = 0; i < 3; i++)
	{
		for(j = 0; j < 3; j++)
			m[i][j] = (FLOAT)d[i][j];
	}
}

inline void TRANSFORMMATRIX::SetIdentity()
{
	_11 = 1.0f;
	_12 = 0.0f;
	_13 = 0.0f;
	_21 = 0.0f;
	_22 = 1.0f;
	_23 = 0.0f;
	_31 = 0.0f;
	_32 = 0.0f;
	_33 = 1.0f;
}

inline void TRANSFORMMATRIX::MultiplyLeft(TRANSFORMMATRIX* pMatrix)
{
	TRANSFORMMATRIX Matrix;
	DWORD i;
	DWORD j;
	Matrix = *this;
	for(i = 0; i < 3; i++)
	{
		for(j = 0; j < 3; j++)
			m[i][j] = pMatrix->m[i][0] * Matrix.m[0][j] + pMatrix->m[i][1] * Matrix.m[1][j] + pMatrix->m[i][2] * Matrix.m[2][j];
	}
}

inline void TRANSFORMMATRIX::MultiplyRight(TRANSFORMMATRIX* pMatrix)
{
	TRANSFORMMATRIX Matrix;
	DWORD i;
	DWORD j;
	Matrix = *this;
	for(i = 0; i < 3; i++)
	{
		for(j = 0; j < 3; j++)
			m[i][j] = Matrix.m[i][0] * pMatrix->m[0][j] + Matrix.m[i][1] * pMatrix->m[1][j] + Matrix.m[i][2] * pMatrix->m[2][j];
	}
}

//	ʂ̏\܂B
//	Effect͌ʂ̔ԍ܂B
//	HostVersion͑ΉVisual Clipper̃o[W܂B
//	EffectVersion͌ʂ̃o[W܂B
//	Flags͌ݎgpĂ܂B
//	Function͌ݎgpĂ܂B
//	Reserved͌ݎgpĂ܂B
//	Name͌ʂ̖O܂B
//	Copyright͌ʂ̒쌠܂B
//	Description͌ʂ̐܂B
//	NameACopyrightADescription̖̕[NULLłȂ΂Ȃ܂B
struct EFFECTINFO
{
	DWORD Effect;
	union
	{
		DWORD HostVersion;
		struct
		{
			WORD MinorHostVersion;
			WORD MajorHostVersion;
		};
	};
	union
	{
		DWORD EffectVersion;
		struct
		{
			WORD MinorEffectVersion;
			WORD MajorEffectVersion;
		};
	};
	DWORD Flags;
	void* Function;
	DWORD Reserved;
	WCHAR Name[64];
	WCHAR Copyright[64];
	WCHAR Description[256];
};

//	TransformEffectBlendModeɊi[l܂B
#define BLEND_MODE_DEFAULT					0
#define BLEND_MODE_AND						1
#define BLEND_MODE_OR						2
#define BLEND_MODE_XOR						3
#define BLEND_MODE_MULTIPLY					4
#define BLEND_MODE_ADD						5
#define BLEND_MODE_SUBTRACT					6
#define BLEND_MODE_INVALID					7

//	ȉ̊֐GNX|[gĂB
BOOL WINAPI DllMain(HANDLE hinstDLL, DWORD fdwReason, LPVOID lpvReserved);
BOOL CALLBACK QueryEffect(DWORD Effect, EFFECTINFO* pEffectInfo);
DWORD CALLBACK EnumEffect(EFFECTINFO* pEffectInfo);
BOOL CALLBACK TransformEffect(DWORD Effect, void* pDestBits, DWORD DestPitch, DWORD DestHeight, RECT* pDestRect, RECT* pSrcRect, COLORREF Color, DWORD BlendMode, TRANSFORMMATRIX* pMatrix, TRANSFORMMATRIX* pInverse);

